#include "EMA_Simple.mqh"

#define MIN_INTERVAL 16000 // GetTickCount

class TIMER
{
private:
  ulong PrevTime;
  ulong NextTime;
  int IntervalTime;
  EMA_SIMPLE RealTimerCycle;
  bool PauseMode;

protected:
  int ChangeVelocity( const int Amount, const double Koef, const int MinValue = 1, const int MaxValue = 128 )
  {
    const int MinInterval = (int)this.RealTimerCycle.Get();
    const double c = Koef * Amount / this.IntervalTime;

    int Res = Amount;

    if (Koef > 0)
    {
      Res = ::MathMin(::MathMax((int)(MinInterval * c), MinValue), MaxValue);

      this.IntervalTime = (int)(Res / c);

      if (this.IntervalTime < MinInterval)
      {
        if ((Koef < 1) ? (Res > MinValue) : (Res < MaxValue))
          this.IntervalTime = ::MathMin((int)((++Res) / c), MinInterval);
        else
          this.IntervalTime = MinInterval;
      }
    }

    return(Res);
  }

  bool IsMyTime( void )
  {
    const ulong TmpTime = ::GetMicrosecondCount();
    const bool Res = !this.PauseMode && (TmpTime > this.NextTime);

    if (!this.PauseMode)
    {
      this.RealTimerCycle.Set(TmpTime - this.PrevTime);

      if (Res)
        this.NextTime = (TmpTime / this.IntervalTime + 1) * this.IntervalTime;
    }

    this.PrevTime = TmpTime;

    return(Res);
  }

public:
  TIMER( const int EMAPeriod = 10 ) : PrevTime(::GetMicrosecondCount()),
                                      NextTime(PrevTime),
                                      IntervalTime(MIN_INTERVAL),
                                      RealTimerCycle(EMAPeriod),
                                      PauseMode(false)
  {
    this.RealTimerCycle.Set(this.IntervalTime);
  }

  bool ChangePauseMode( void )
  {
    return(this.PauseMode = !this.PauseMode);
  }

  bool GetPauseMode( void ) const
  {
    return(this.PauseMode);
  }
};

#undef MIN_INTERVAL